/*
Module : PerfCtrv2Mgr.h
Purpose: Defines the interface for a set of C++ wrapper classes for the Perf Counter v2 framework 
         available in Windows Vista and later
Created: PJN / 11-10-2012
History: PJN / 02-01-2016 1. Updated copyright details
                          2. Added SAL annotations to all the code
                          3. Removed unnecessary CStringW instance from CPerfProvider::QueryInstance method
                          4. CPerfCounterSet::CreateInstance now uses a PCWSTR parameter instead of a 
                          LPCTSTR parameter. This avoids the need to use of CStringW local variable
                          5. Updated the parameter values names for the classes to be consistent with the 
                          Windows 10 SDK perflib.h header.
                          6. Updated the code to compile without taking a dependency on MFC.
         PJN / 10-02-2016 1. Updated the code to use LoadLibrary rather than relying on client applications
                          already having a implicit dependency on ADVAPI32.dll. This bug / issue was 
                          spotted when you compile the sample app in release mode which ships with the 
                          wrapper classes.
         PJN / 19-09-2018 1. Updated copyright details.
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean 
                          that the code will now only compile on VC 2017 or later.
                          3. Removed all code which uses LoadLibrary \ GetProcAddress
         PJN / 12-04-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 21-09-2021 1. Reinstated a previous GetInstance method
         PJN / 17-03-2022 1. Updated copyright details
                          2. Updated the code to use C++ uniform initialization for all variable declarations.

Copyright (c) 2012 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


////////////////////////// Macros / Defines ///////////////////////////////////

#pragma once

#ifndef __PERFCTRV2MGR_H__
#define __PERFCTRV2MGR_H__


///////////////////////// Includes ////////////////////////////////////////////

#ifndef _PERFLIB_H_
#pragma message("To avoid this message, please put perflib.h in your pre compiled header (normally stdafx.h)")
#include <perflib.h>
#endif //#ifndef _PERFLIB_H_

#ifndef __ATLBASE_H__
#pragma message("To avoid this message, please put atlbase.h in your pre compiled header (normally stdafx.h)")
#include <atlbase.h>
#endif //#ifndef __ATLBASE_H__


///////////////////////// Classes /////////////////////////////////////////////

//Class wrapper for a perf provider
class CPerfProvider
{
public:
//Constructors / Destructors
  CPerfProvider() noexcept : m_hProvider{nullptr}
  {
  }

  CPerfProvider(_In_ const CPerfProvider&) = delete;
  CPerfProvider(_In_ CPerfProvider&&) = delete;

  ~CPerfProvider() noexcept
  {
    Stop();
  }

//Methods
  CPerfProvider& operator=(_In_ const CPerfProvider&) = delete;
  CPerfProvider& operator=(_In_ CPerfProvider&&) = delete;

  ULONG Start(_In_ LPGUID Guid, _In_opt_ PERFLIBREQUEST ControlCallback = nullptr) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_hProvider == nullptr);

    return PerfStartProvider(Guid, ControlCallback, &m_hProvider);
  }

  ULONG StartEx(_In_ LPGUID Guid, _In_opt_ PPERF_PROVIDER_CONTEXT ProviderContext) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_hProvider == nullptr);

    //Call through the function pointer
    return PerfStartProviderEx(Guid, ProviderContext, &m_hProvider);
  }

  ULONG StartEx(_In_ LPGUID Guid, _In_opt_ PERFLIBREQUEST NotificationCallback = nullptr, _In_opt_ PERF_MEM_ALLOC MemoryAllocationFunction = nullptr,
                _In_opt_ PERF_MEM_FREE MemoryFreeFunction = nullptr, _In_opt_ PVOID MemoryFunctionsContext = nullptr) noexcept
  {
    //Setup an appropriate PERF_PROVIDER_CONTEXT
    PERF_PROVIDER_CONTEXT ProviderContext{};
    ProviderContext.ContextSize = sizeof(ProviderContext);
    ProviderContext.ControlCallback = NotificationCallback;
    ProviderContext.MemAllocRoutine = MemoryAllocationFunction;
    ProviderContext.MemFreeRoutine = MemoryFreeFunction;
    ProviderContext.pMemContext = MemoryFunctionsContext;

    //Delegate to the other version of "StartEx"
    return StartEx(Guid, &ProviderContext);
  }

  ULONG Stop() noexcept
  {
    ULONG nReturn{ERROR_INVALID_HANDLE};
    if (m_hProvider)
    {
      nReturn = PerfStopProvider(m_hProvider);
      m_hProvider = nullptr;
    }

    return nReturn;
  }

  _NODISCARD PPERF_COUNTERSET_INSTANCE QueryInstance(_In_ LPCGUID Guid, _In_ PCWSTR Name, _In_ ULONG Id) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_hProvider != nullptr);

    return PerfQueryInstance(m_hProvider, Guid, Name, Id);
  }

  ULONG SetCounterSetInfo(_Inout_updates_bytes_(TemplateSize) PPERF_COUNTERSET_INFO Template, _In_ ULONG TemplateSize) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_hProvider != nullptr);

    return PerfSetCounterSetInfo(m_hProvider, Template, TemplateSize);
  }

protected:
//Member variables
  HANDLE m_hProvider; //The provider handle we are wrapping

  friend class CPerfCounterSet; //Allows this class to use the internal cached function pointers we maintain
};


//Class wrapper for a perf counter set instance
class CPerfCounterSet
{
public:
//Constructors / Destructors
  CPerfCounterSet() noexcept : m_pInstance{nullptr},
                               m_pProvider{nullptr}
  {
  }

  CPerfCounterSet(const CPerfCounterSet&) = delete;
  CPerfCounterSet(CPerfCounterSet&&) = delete;

  ~CPerfCounterSet() noexcept
  {
    //Free up the instance if valid
    if (m_pInstance != nullptr)
      DeleteInstance();
  }

//Methods
  CPerfCounterSet& operator=(const CPerfCounterSet&) = delete;
  CPerfCounterSet& operator=(CPerfCounterSet&&) = delete;

  void Initialize(_In_ CPerfProvider* pProvider) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider == nullptr);
    m_pProvider = pProvider;
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);
  }

  _NODISCARD BOOL Initialized() const noexcept
  {
    return (m_pProvider != nullptr);
  }

  _NODISCARD PPERF_COUNTERSET_INSTANCE GetInstance() noexcept
  {
    return m_pInstance;
  }

  ULONG CreateInstance(_In_ LPCGUID Guid, _In_ PCWSTR Name, _In_ ULONG Id) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance == nullptr);

    //What will be the return value from this function
    ULONG nReturn{ERROR_SUCCESS};

    m_pInstance =  PerfCreateInstance(m_pProvider->m_hProvider, Guid, Name, Id);
    if (m_pInstance == nullptr)
      nReturn = GetLastError();

    return nReturn;
  }

  ULONG DeleteInstance() noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);

    //What will be the return value from this function
    ULONG nReturn{ERROR_SUCCESS};

    nReturn = PerfDeleteInstance(m_pProvider->m_hProvider, m_pInstance);
    m_pInstance = nullptr;

    return nReturn;
  }

  ULONG IncrementL(_In_ ULONG CounterId, _In_ ULONG Value) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfIncrementULongCounterValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Value);
  }

  ULONG IncrementLL(_In_ ULONG CounterId, _In_ ULONGLONG Value) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfIncrementULongLongCounterValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Value);
  }

  ULONG SetL(_In_ ULONG CounterId, _In_ ULONG Value) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfSetULongCounterValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Value);
  }

  ULONG SetLL(_In_ ULONG CounterId, _In_ ULONGLONG Value) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfSetULongLongCounterValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Value);
  }

  ULONG DecrementL(_In_ ULONG CounterId, _In_ ULONG Value) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfDecrementULongCounterValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Value);    
  }

  ULONG DecrementLL(_In_ ULONG CounterId, _In_ ULONGLONG Value) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfDecrementULongLongCounterValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Value);    
  }

  ULONG SetRef(_In_ ULONG CounterId, _In_ PVOID Address) noexcept
  {
    //Validate our parameters
#pragma warning(suppress: 26477)
    ATLASSERT(m_pInstance != nullptr);
#pragma warning(suppress: 26477)
    ATLASSUME(m_pProvider != nullptr);
#pragma warning(suppress: 26477)
    ATLASSERT(m_pProvider->m_hProvider != nullptr);

    return PerfSetCounterRefValue(m_pProvider->m_hProvider, m_pInstance, CounterId, Address);    
  }

protected:
//Member variables
  PPERF_COUNTERSET_INSTANCE m_pInstance;
  CPerfProvider* m_pProvider;
};

#endif //#ifndef __PERFCTRV2MGR_H__
